const async = require('async');
const chalk = require('chalk');
const fs = require('fs');
const path = require('path');
const shortcode = require('../utils/shortcode');
const showdown = require('showdown');
const markdown = new showdown.Converter({
  extensions: [shortcode]
});
const utils = require('../utils');
const request = require('request');
const ejs = require('ejs');
const he = require('he');
const MARKDOWN_EXTENSION = '.md';

module.exports = (section, meta, callback) => {

  // Task flow control
  async.parallel([

    // Compile content for top level page
    async.apply((utils.isExternal(section.contentSrc)) ? getExternalContent : getContent, section),

    // Compile content for children if required
    (cb) => async.map(section.children, (page, cb) => {
      if (utils.isExternal(page.contentSrc)) {
        getExternalContent(page, cb);
      } else {
        getContent(page, cb);
      }
    }, cb)

  ], done);

  ///////////////////////////////////////////////////////////////////////
  // TO MOVE [START]
  ///////////////////////////////////////////////////////////////////////

  /**
   * Return snippet template
   */
  function getSnippet(str, data) {
    var filename = path.join(meta.snippets, str, 'template.ejs');

    // Compile template if it exists
    if (fs.existsSync(filename)) {
      var templateStr = fs.readFileSync(filename, 'utf-8');

      return ejs.render(templateStr, {
        styles: meta.cssMap,
        data: data
      }, {
        filename: filename,
        cache: false
      });

    } else {
      return str;
    }
  }

  function compileSnippet(args) {
    var hasExtension = path.extname(args[0]);
    var template = '';

    // Handle array of snippets
    if (args[0].constructor === Array) {

      var templateArr = args[0].map((item) => {
        return getSnippet(item.template, item.data);
      });

      template = templateArr.join('\n\n');

    } else {

      // Handle individual snippet
      var templateStr = args[0];
      var templateData = args[1] || {};
      template = getSnippet(templateStr, templateData);

    }

    return template;
  }

  // Compile snippet template
  var code = function () {
    // Render snippet into snippetTemplate
    return ejs.render(meta.snippetTemplate, {
      styles: meta.cssMap,
      isolateScopeCssWrapperClass: meta.isolateScopeCssWrapperClass,
      isolateScopeCssClass: meta.isolateScopeCssClass,
      template: compileSnippet(arguments),
      encode: he.encode
    }, {
      cache: false
    });
  };

  // Render snippet
  var render = function (str, data) {

    // Render snippet into renderTemplate
    return ejs.render(meta.renderTemplate, {
      styles: meta.cssMap,
      isolateScopeCssWrapperClass: meta.isolateScopeCssWrapperClass,
      isolateScopeCssClass: meta.isolateScopeCssClass,
      template: compileSnippet(arguments)
    }, {
      cache: false
    });
  };

  ///////////////////////////////////////////////////////////////////////
  // TO MOVE [END]
  ///////////////////////////////////////////////////////////////////////

  // On complete handler
  function done(err, result) {
    if (err) {
      callback(err);
    } else {
      callback(null, section);
    }
  }

  /**
   * Do any additional content transformation
   * based on file extension
   */
  function compileContent(content, fileExtension) {
    var _content;

    switch (fileExtension) {
    case MARKDOWN_EXTENSION:
      _content = markdown.makeHtml(content);
      break;
    default:
      _content = content;
    }

    // Always run tempates through ejs parser
    // as code snippets may need to be rendered here
    return ejs.render(_content, {
      render: render, // render snippet method
      code: code, // code preview method
      styles: meta.cssMap
    }, {
      cache: false
    });
  }

  /**
   * Wrap content in specified wrapper template parse
   * through ejs engine
   */
  function wrapContent(content, sectionId) {
    return ejs.render(meta.contentWrapperTemplate, {
      content: content,
      id: sectionId,
      styles: meta.cssMap
    }, {
      cache: false
    });
  }

  function getContent(page, cb) {

    if (page.processor) {

      page.processorOutput.map((data, index) => {
        var contentKey = `${page.key}-${index}`;
        page.contentCompiled.push(wrapContent(compileContent(data), contentKey));
      });

      cb();

    } else {
      async.forEachOf(page.fileDependencies, (src, index, cb) => {
        fs.readFile(src, 'utf-8', (err, data) => {
          if (err) {
            cb(err);
          } else {
            var contentKey = `${page.key}-${index}`;
            var fileExtension = path.extname(src);

            page.contentCompiled[index] = wrapContent(compileContent(data, fileExtension), contentKey);

            cb();
          }
        });
      }, cb);
    }
  }

  function getExternalContent(page, cb) {
    request(page.contentSrc, function (error, response, data) {
      if (!error && response.statusCode === 200) {
        var contentKey = `${page.key}`;
        var fileExtension = path.extname(page.contentSrc);
        page.contentCompiled.push(wrapContent(compileContent(data, fileExtension), contentKey));
        cb();
      } else {
        console.warn(chalk.red(
          `Could not retrieve content from '${page.contentSrc}'. Please check the url/internet connection and try again.`
        ));
        console.info(chalk.green(
          `Will continue to build site without content for section titled '${page.title}'.`
        ));
        cb();
      }
    });
  }

};
